@@ -97,6 +97,7 @@ from scalyr_agent.platform_controller import (
     CannotExecuteAsUser,
 )
 from scalyr_agent.platform_controller import AgentNotRunning
+from scalyr_agent.build_info import get_build_revision
 
 
 STATUS_FILE = "last_status"
@@ -148,6 +149,10 @@ class ScalyrAgent(object):
         self.__default_paths = platform_controller.default_paths
 
         self.__config_file_path = None
+
+        # An extra directory for config snippets
+        self.__extra_config_dir = None
+
         # If the current contents of the configuration file has errors in it, then this will be set to the config
         # object produced by reading it.
         self.__current_bad_config = None
@@ -244,6 +249,10 @@ class ScalyrAgent(object):
         self.__no_fork = command_options.no_fork
         no_check_remote = False
 
+        self.__extra_config_dir = Configuration.get_extra_config_dir(
+            command_options.extra_config_dir
+        )
+
         # We process for the 'version' command early since we do not need the configuration file for it.
         if command == "version":
             print("The Scalyr Agent 2 version is %s" % SCALYR_VERSION)
@@ -362,7 +371,12 @@ class ScalyrAgent(object):
         @return: The configuration object.
         @rtype: scalyr_agent.Configuration
         """
-        return Configuration(config_file_path, self.__default_paths, log)
+        return Configuration(
+            config_file_path,
+            self.__default_paths,
+            log,
+            extra_config_dir=self.__extra_config_dir,
+        )
 
     def __verify_config(
         self,
@@ -907,19 +921,23 @@ class ScalyrAgent(object):
                     logs_initial_positions = None
 
                 # 2->TODO it was very helpful to see what python version does agent run on. Maybe we can keep it?
-
                 python_version_str = sys.version.replace("\n", "")
+                build_revision = get_build_revision()
 
-                log.info(
-                    "Starting scalyr agent... (version=%s) %s (Python version: %s)"
-                    % (SCALYR_VERSION, scalyr_util.get_pid_tid(), python_version_str)
-                )
-                log.log(
-                    scalyr_logging.DEBUG_LEVEL_1,
-                    "Starting scalyr agent... (version=%s) %s (Python version: %s)"
-                    % (SCALYR_VERSION, scalyr_util.get_pid_tid(), python_version_str),
+                # TODO: Why do we log the same line under info and debug? Intentional?
+                msg = (
+                    "Starting scalyr agent... (version=%s) (revision=%s) %s (Python version: %s)"
+                    % (
+                        SCALYR_VERSION,
+                        build_revision,
+                        scalyr_util.get_pid_tid(),
+                        python_version_str,
+                    )
                 )
 
+                log.info(msg)
+                log.log(scalyr_logging.DEBUG_LEVEL_1, msg)
+
                 self.__controller.emit_init_log(log, self.__config.debug_init)
 
                 self.__start_or_stop_unsafe_debugging()
@@ -928,6 +946,8 @@ class ScalyrAgent(object):
                 raw_scalyr_server = self.__config.raw_scalyr_server
                 self.__print_force_https_message(scalyr_server, raw_scalyr_server)
 
+                self.__config.print_useful_settings()
+
                 self.__scalyr_client = self.__create_client()
 
                 def start_worker_thread(config, logs_initial_positions=None):
@@ -948,7 +968,13 @@ class ScalyrAgent(object):
                 # JSON library setting is applied as part of __create_worker_thread method
                 log.log(
                     scalyr_logging.DEBUG_LEVEL_0,
-                    "JSON library is %s" % (scalyr_util.get_json_lib()),
+                    'Using JSON library "%s"' % (scalyr_util.get_json_lib()),
+                )
+
+                log.log(
+                    scalyr_logging.DEBUG_LEVEL_0,
+                    'Using "%s" compression algorithm with level "%s"'
+                    % (self.__config.compression_type, self.__config.compression_level),
                 )
 
                 current_time = time.time()
@@ -1170,6 +1196,8 @@ class ScalyrAgent(object):
 
                     worker_thread = None
 
+                    new_config.print_useful_settings(self.__config)
+
                     self.__config = new_config
                     self.__controller.consume_config(new_config, new_config.file_path)
 
@@ -1421,6 +1449,7 @@ class ScalyrAgent(object):
         result = AgentStatus()
         result.launch_time = self.__start_time
         result.user = self.__controller.get_current_user()
+        result.revision = get_build_revision()
         result.version = SCALYR_VERSION
         result.server_host = self.__config.server_attributes["serverHost"]
         result.scalyr_server = self.__config.scalyr_server
@@ -1734,6 +1763,12 @@ if __name__ == "__main__":
         help="Read configuration from FILE",
         metavar="FILE",
     )
+    parser.add_option(
+        "--extra-config-dir",
+        default=None,
+        help="An extra directory to check for configuration files",
+        metavar="PATH",
+    )
     parser.add_option(
         "-q",
         "--quiet",
