@@ -1,3 +1,17 @@
+# Copyright 2014-2020 Scalyr Inc.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#   http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 from __future__ import absolute_import
 
 if False:
@@ -6,6 +20,7 @@ if False:
 import sys
 import struct
 import os
+import subprocess
 
 import six
 
@@ -181,3 +196,17 @@ def find_executable(executable):
         return which(executable)
 
     return distutils_find_executable(executable)
+
+
+def subprocess_check_output(cmd, *args, **kwargs):
+    """
+    Wrapper around subprocess.check_output which is not available under Python 2.6.
+    """
+    if sys.version_info < (2, 7, 0):
+        output = subprocess.Popen(
+            cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, *args, **kwargs
+        ).communicate()[0]
+    else:
+        output = subprocess.check_output(cmd, *args, **kwargs)
+
+    return output
